package com.psol.xcommerce;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * represents a list of merchants
 *
 * @version Sep 10, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class MerchantCollection
   extends Comlet
{
   /**
    * properties
    */
   protected Dictionary merchants = new Hashtable();
   protected Document merchantsDocument = null,
                      merchantsXSL = null;

   /**
    * creates a new Merchant object
    * @param shop the shop it is part of
    * @param ServletException error reading the merchant list
    */
   public MerchantCollection(Shop shop)
      throws ServletException
   {
      super(shop);
      String fname = getInitParameter("merchants.xml");
      if(null != fname)
         merchantsDocument = XMLUtil.parse(fname);
      if(null == merchantsDocument)
         throw new UnavailableException(shop,"merchants.xml");
      freshened();
      Element topLevel = merchantsDocument.getDocumentElement();
      Enumeration enumeration =
          XMLUtil.extract(topLevel,"merchant");
      while(enumeration.hasMoreElements())
      {
         Element element =
            (Element)enumeration.nextElement();
         Merchant merchant =
            new Merchant(element,shop);
         merchants.put(merchant.getID(),merchant);
      }
   }

   /**
    * return a document with the list of merchants
    */
   protected Document getDocument()
   {
      return merchantsDocument;
   }

   /**
    * return the style sheet for the list of merchants
    * @exception ServletException error reading the document
    */
   protected Document getXSL()
      throws ServletException
   {
      if(null == merchantsXSL)
      {
         String fname = getInitParameter("merchants.xsl");
         if(null != fname)
            merchantsXSL = XMLUtil.parse(fname);
      }
      return merchantsXSL;
   }

   /**
    * return a given merchant.
    * @param id merchant id
    */
   public Merchant getMerchant(String id)
   {
      return (Merchant)merchants.get(id);
   }

   /**
    * handle GET request
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   public void doGet(HttpServletRequest request,
                     HttpServletResponse response)
      throws IOException, ServletException
   {
      XMLUtil.transform(getDocument(),
                        getXSL(),
                        response.getWriter(),
                        response.getCharacterEncoding());
   }
}