package com.psol.xcommerce;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * a merchant is a collection of products
 *
 * @version Sep 10, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class Merchant
   extends Comlet
{
   /**
    * properties
    */
   protected Element merchantElement;
   protected Dictionary products = null;
   protected Document productsDocument = null,
                      productsXSL = null,
                      productXSL = null;
   protected String id;
   protected long expire = Long.MAX_VALUE;

   /**
    * creates a new Merchant object
    * @param element the merchant element
    * @param shop the shop it is part of
    */
   public Merchant(Element element,Shop shop)
   {
      super(shop);
      merchantElement = element;
   }

   /**
    * return the element id
    */
   public String getID()
   {
      return merchantElement.getAttribute("id");
   }

   /**
    * return the post URL
    */
   public String getPostURL()
   {
      Element postElement =
         XMLUtil.extractFirst(merchantElement,"post");
      if(null != postElement)
         return postElement.getAttribute("href");
      else
         return null;
   }

   /**
    * return the post user
    */
   public String getPostUser()
   {
      Element postElement =
         XMLUtil.extractFirst(merchantElement,"post");
      if(null != postElement)
         return postElement.getAttribute("user");
      else
         return null;
   }

   /**
    * return the post password
    */
   public String getPostPassword()
   {
      Element postElement =
         XMLUtil.extractFirst(merchantElement,"post");
      if(null != postElement)
         return postElement.getAttribute("password");
      else
         return null;
   }

   /**
    * return the list of products
    * @exception ServletException error reading the document
    */
   protected Document getDocument()
      throws ServletException
   {
      if(null == productsDocument ||
         expire < System.currentTimeMillis())
      {
         Element productsElement =
            XMLUtil.extractFirst(merchantElement,"products");
         if(null != productsElement)
         {
            String fname =
               productsElement.getAttribute("href");
            String update =
               productsElement.getAttribute("update");
            if(!XMLUtil.isEmpty(fname))
            {
               productsDocument = XMLUtil.parse(fname);
               freshened();
               productXSL = null;
               productsXSL = null;
            }
            if(!XMLUtil.isEmpty(update))
            {
               long u = Long.parseLong(update) * 1000;
               expire = System.currentTimeMillis() + u;
            }
         }
      }
      return productsDocument;
   }

   /**
    * return the style sheet for a list of products
    * @exception ServletException error reading the document
    */
   protected Document getXSL()
      throws ServletException
   {
      if(null == productsXSL)
      {
         Element productsXSLElement =
            XMLUtil.extractFirst(merchantElement,
                                 "stylesheet-all");
         if(null != productsXSLElement)
         {
            String fname = 
               productsXSLElement.getAttribute("href");
            if(!XMLUtil.isEmpty(fname))
               productsXSL = XMLUtil.parse(fname);
         }
      }
      return productsXSL;
   }

   /**
    * return the style sheet for one product
    * @exception ServletException error reading the document
    */
   public Document getProductXSL()
      throws ServletException
   {
      if(null == productXSL)
      {
         Element productXSLElement =
            XMLUtil.extractFirst(merchantElement,"stylesheet");
         if(null != productXSLElement)
         {
            String fname =
               productXSLElement.getAttribute("href");
            if(!XMLUtil.isEmpty(fname))
               productXSL = XMLUtil.parse(fname);
         }
      }
      return productXSL;
   }

   /**
    * return a given product
    * @param id product index
    */
   public Product getProduct(String id)
      throws ServletException
   {
      if(null == products ||
         expire < System.currentTimeMillis())
      {
         Document productsDocument = getDocument();
         if(null != productsDocument)
         {
            Element topLevel =
               productsDocument.getDocumentElement();
            Enumeration enumeration =
                XMLUtil.extract(topLevel,"product");
            products = new Hashtable();
            while(enumeration.hasMoreElements())
            {
               Element element =
                  (Element)enumeration.nextElement();
               Product product =
                  new Product(element,this,shop);
               products.put(product.getID(),product);
            }
         }
      }
      // re-test: reading the product may fail
      if(null != products)
         return (Product)products.get(id);
      else
         return null;
   }

   /**
    * handle GET request
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   public void doGet(HttpServletRequest request,
                     HttpServletResponse response)
      throws IOException, ServletException
   {
      XMLUtil.transform(getDocument(),
                        getXSL(),
                        response.getWriter(),
                        response.getCharacterEncoding());
   }
}