package com.psol.xcommerce;

import java.io.*;
import java.text.*;
import java.util.*;
import org.w3c.dom.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * presents the invoice and emails it
 *
 * @version Sep 10, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class Checkout
   extends Comlet
{
   /**
    * creates a new Checkout object
    * @param shop the shop it is part of
    */
   public Checkout(Shop shop)
   {
      super(shop);
   }

   /**
    * handle POST request
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   public void doPost(HttpServletRequest request,
                      HttpServletResponse response)
      throws ServletException, IOException
   {
      if(isAddressComplete(request))
         doSaveOrder(request,response);
      else
         doCollectData(request,response);
   }

   /**
    * return true if the address is complete
    * @param request client request
    */
   protected boolean isAddressComplete(HttpServletRequest request)
   {
      // region is not mandatory
      String[] fields =
      {
         "name", "street", "postal-code",
         "locality", "country", "email"
      };
      int found = 0;
      for(int i = 0;i < fields.length;i++)
      {
         String value = request.getParameter(fields[i]);
         if(!XMLUtil.isEmpty(value))
            found++;
      }
      return found == fields.length;
   }

   /**
    * save the order
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   public void doSaveOrder(HttpServletRequest request,
                           HttpServletResponse response)
      throws ServletException, IOException
   {
      String productid = request.getParameter("product"),
             merchantid = request.getParameter("merchant");
      Product product = getProduct(merchantid,productid);
      if(null == product)
      {
         response.sendError(HttpServletResponse.SC_NOT_FOUND);
         return;
      }
      Merchant merchant = product.getMerchant();
      String postURL = merchant.getPostURL();
      Writer writer = null;
      if(null != postURL)
         writer = new StringWriter();
      else
      {
         String directory = getInitParameter(merchant.getID()
                                             + ".orders"),
                // should be enough to avoid duplicates
                fname = String.valueOf(System.currentTimeMillis())
                                       + ".xml";
         File file = new File(directory,fname);
         writer = new FileWriter(file);
      }
      writer.write("<?xml version=\"1.0\"?>");
      writer.write("<order>");
      writer.write("<buyer");
      writeAttribute("name",request,writer);
      writeAttribute("street",request,writer);
      writeAttribute("region",request,writer);
      writeAttribute("postal-code",request,writer);
      writeAttribute("locality",request,writer);
      writeAttribute("country",request,writer);
      writeAttribute("email",request,writer);
      writer.write("/>");
      writer.write("<product");
      writeAttribute("quantity",request,writer);
      writer.write(" id=\"");
      writer.write(product.getID());
      writer.write("\" name=\"");
      writer.write(product.getName());
      writer.write("\" price=\"");
      writer.write(product.getPrice());
      writer.write("\"/></order>");
      writer.close();
      if(null != postURL)
      {
         Dictionary parameters = new Hashtable();
         String user = merchant.getPostUser(),
                password = merchant.getPostPassword(),
                xmlData = writer.toString();
         parameters.put("user",user);
         parameters.put("password",password);
         parameters.put("xmldata",xmlData);
         HTTPPost post = new HTTPPost(postURL,parameters);
         post.doRequest();
      }
      writer = response.getWriter();
      writer.write("<HTML><HEAD><TITLE>Checkout</TITLE></HEAD>");
      writer.write("<BODY><P>Thank you for shopping with us!");
      writer.write("<BR><A HREF=\"");
      writer.write(request.getServletPath());
      writer.write("\">Return to the shop</A>");
      writer.write("</BODY></HTML>");
      writer.flush();
   }

   /**
    * helper method: return the product
    */
   protected Product getProduct(String merchantid,
                                String productid)
      throws ServletException
   {
       MerchantCollection merchants = shop.getMerchants();
       Merchant merchant = merchants.getMerchant(merchantid);
       if(null != merchant)
          return merchant.getProduct(productid);
       else
          return null;
   }

   /**
    * helper method: write one attribute
    */
   protected void writeAttribute(String id,
                                 HttpServletRequest request,
                                 Writer writer)
      throws IOException
   {
      String value = request.getParameter(id);
      if(!XMLUtil.isEmpty(value))
      {
         writer.write(" ");
         writer.write(id);
         writer.write("=\"");
         writer.write(value);
         writer.write("\"");
      }
   }

   /**
    * collect buyer data
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   public void doCollectData(HttpServletRequest request,
                             HttpServletResponse response)
      throws ServletException, IOException
   {
      String productid = request.getParameter("product"),
             merchantid = request.getParameter("merchant"),
             quantity = request.getParameter("quantity");
      Product product = getProduct(merchantid,productid);
      if(null == product)
      {
         response.sendError(HttpServletResponse.SC_NOT_FOUND);
         return;
      }
      Writer writer = response.getWriter();
      writer.write("<HTML><HEAD><TITLE>Checkout</TITLE></HEAD>");
      writer.write("<BODY><P>Enter your name and address:");
      writer.write("<FORM METHOD=\"POST\" ACTION=\"");
      writer.write(request.getServletPath() + "/checkout");
      writer.write("\"><TABLE BORDER=\"0\">");
      writeRow("Name *:","name",request,writer);
      writeRow("Street *:","street",request,writer);
      writeRow("Region:","region",request,writer);
      writeRow("ZIP or postal-code *:","postal-code",request,writer);
      writeRow("Locality *:","locality",request,writer);
      writeRow("Country *:","country",request,writer);
      writeRow("Email *:","email",request,writer);
      writer.write("</TABLE>");
      writer.write("Your order: ");
      writer.write(quantity);
      writer.write(" * ");
      writer.write(product.getName());
      writer.write(" at ");
      writer.write(product.getPrice());
      writer.write(" each<BR>");
      writer.write("<INPUT TYPE=\"HIDDEN\" NAME=\"product\"");
      writer.write(" VALUE=\"");
      writer.write(productid);
      writer.write("\">");
      writer.write("<INPUT TYPE=\"HIDDEN\" NAME=\"merchant\"");
      writer.write(" VALUE=\"");
      writer.write(merchantid);
      writer.write("\">");
      writer.write("<INPUT TYPE=\"HIDDEN\" NAME=\"quantity\"");
      writer.write(" VALUE=\"");
      writer.write(quantity);
      writer.write("\">");
      writer.write("<INPUT TYPE=\"SUBMIT\" VALUE=\"Order\">");
      writer.write("</FORM></HTML>");
      writer.flush();
   }

   /**
    * helper method: write one row
    */
   protected void writeRow(String label,
                           String id,
                           HttpServletRequest request,
                           Writer writer)
      throws IOException
   {
      writer.write("<TR><TD>");
      writer.write(label);
      writer.write("</TD><TD>");
      writer.write("<INPUT TYPE=\"TEXT\" NAME=\"");
      writer.write(id);
      writer.write("\"");
      String value = request.getParameter(id);
      if(!XMLUtil.isEmpty(value))
      {
         writer.write(" VALUE=\"");
         writer.write(value);
         writer.write("\"");
      }
      writer.write("></TD></TR>");
   }
}