package com.psol.xcommerce;

import java.io.*;
import org.w3c.dom.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Viewer is a web-tool to view orders
 * for smaller merchants.
 *
 * @version Sep 10, 19999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class Viewer
   extends HttpServlet
{
   /**
    * the viewer's style sheet
    */
   protected Document styleSheet;

   /**
    * initializes the servlet, read the style sheet
    * @exception could not read the style sheet
    */
   public void init()
      throws ServletException
   {
      String fname = getInitParameter("viewer.xsl");
      styleSheet = XMLUtil.parse(fname);
   }

   /**
    * process GET requests
    * @param request request received from the client
    * @param response response to the client
    */
   protected void doGet(HttpServletRequest request,
                        HttpServletResponse response)
      throws IOException, ServletException
   {
      String merchant = request.getParameter("merchant"),
             path = getInitParameter(merchant + ".orders"),
             fname = request.getParameter("fname");
      if(null == merchant || null == path)
      {
         response.sendError(HttpServletResponse.SC_NOT_FOUND);
         return;
      }
      if(null == fname)
      {
         File file = new File(path);
         String[] files = file.list();
         Writer writer = response.getWriter();
         writer.write("<HTML><HEAD><TITLE>Order list</TITLE>");
         writer.write("</HEAD><BODY><UL>");
         for(int i = 0;i < files.length;i++)
         {
            writer.write("<LI><A HREF=\"");
            writer.write(request.getServletPath());
            writer.write("?merchant=");
            writer.write(merchant);
            writer.write("&fname=");
            writer.write(files[i]);
            writer.write("\">");
            writer.write(files[i]);
            writer.write("</A></LI>");
         }
         writer.write("</UL></BODY></HTML>");
         writer.flush();
      }
      else
      {
         File file = new File(path,fname);
         Document document = XMLUtil.parse(file.getPath());
         XMLUtil.transform(document,
                           styleSheet,
                           response.getWriter(),
                           response.getCharacterEncoding());
      }
   }
}